# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: objectstore/definition.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from unionai.internal.validate.validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1cobjectstore/definition.proto\x12\x17\x63loudidl.objectstore.v1\x1a\x17validate/validate.proto\"\x80\x01\n\x08Metadata\x12<\n\x03tag\x18\x02 \x03(\x0b\x32*.cloudidl.objectstore.v1.Metadata.TagEntryR\x03tag\x1a\x36\n\x08TagEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"$\n\x06Object\x12\x1a\n\x08\x63ontents\x18\x01 \x01(\x0cR\x08\x63ontents\"4\n\x03Key\x12-\n\x03key\x18\x01 \x01(\tB\x1b\xfa\x42\x18r\x16\x10\x01\x32\x12^union(meta)?://.+R\x03key\"4\n\x11PresignPutRequest\x12\x1f\n\x0b\x63ontent_md5\x18\x01 \x01(\x0cR\ncontentMd5\"\x13\n\x11PresignGetRequest*/\n\nHTTPMethod\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x07\n\x03GET\x10\x01\x12\x07\n\x03PUT\x10\x02\x42\xde\x01\n\x1b\x63om.cloudidl.objectstore.v1B\x0f\x44\x65\x66initionProtoH\x02P\x01Z.github.com/unionai/cloud/gen/pb-go/objectstore\xa2\x02\x03\x43OX\xaa\x02\x17\x43loudidl.Objectstore.V1\xca\x02\x17\x43loudidl\\Objectstore\\V1\xe2\x02#Cloudidl\\Objectstore\\V1\\GPBMetadata\xea\x02\x19\x43loudidl::Objectstore::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'objectstore.definition_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\033com.cloudidl.objectstore.v1B\017DefinitionProtoH\002P\001Z.github.com/unionai/cloud/gen/pb-go/objectstore\242\002\003COX\252\002\027Cloudidl.Objectstore.V1\312\002\027Cloudidl\\Objectstore\\V1\342\002#Cloudidl\\Objectstore\\V1\\GPBMetadata\352\002\031Cloudidl::Objectstore::V1'
  _METADATA_TAGENTRY._options = None
  _METADATA_TAGENTRY._serialized_options = b'8\001'
  _KEY.fields_by_name['key']._options = None
  _KEY.fields_by_name['key']._serialized_options = b'\372B\030r\026\020\0012\022^union(meta)?://.+'
  _globals['_HTTPMETHOD']._serialized_start=380
  _globals['_HTTPMETHOD']._serialized_end=427
  _globals['_METADATA']._serialized_start=83
  _globals['_METADATA']._serialized_end=211
  _globals['_METADATA_TAGENTRY']._serialized_start=157
  _globals['_METADATA_TAGENTRY']._serialized_end=211
  _globals['_OBJECT']._serialized_start=213
  _globals['_OBJECT']._serialized_end=249
  _globals['_KEY']._serialized_start=251
  _globals['_KEY']._serialized_end=303
  _globals['_PRESIGNPUTREQUEST']._serialized_start=305
  _globals['_PRESIGNPUTREQUEST']._serialized_end=357
  _globals['_PRESIGNGETREQUEST']._serialized_start=359
  _globals['_PRESIGNGETREQUEST']._serialized_end=378
# @@protoc_insertion_point(module_scope)
